@@ -11,6 +11,7 @@ from typing import Any, Self
 import pytest
 import urllib3
 from selenium.common.exceptions import (
+    ElementClickInterceptedException,
     NoSuchElementException,
     StaleElementReferenceException,
     TimeoutException,
@@ -186,7 +187,10 @@ class Page:
         self, driver: WebDriver, monkeypatch: pytest.MonkeyPatch = None
     ) -> None:
         self._driver = driver
-        self.previous_window_handle = self.driver.current_window_handle
+        if not hasattr(self.driver, "previous_window_handle"):
+            self.driver.previous_window_handle = (
+                self.driver.current_window_handle
+            )
         self.monkeypatch = monkeypatch
 
     @property
@@ -291,13 +295,23 @@ class Page:
     def switch_to_tab(self) -> None:
         """Switch to a newly opened browser tab."""
         new_window = wait_until_new_window_is_opened(
-            self.driver, self.previous_window_handle
+            self.driver, self.driver.previous_window_handle
         )
         switch_to_window(self.driver, new_window)
 
     def switch_previous_tab(self) -> None:
         """Switch back to the previously active tab."""
-        switch_to_window(self.driver, self.previous_window_handle)
+        switch_to_window(self.driver, self.driver.previous_window_handle)
+
+    def open_tab(self) -> None:
+        """Open new tab and switch to it."""
+        self.driver.execute_script("window.open();")
+        self.switch_to_tab()
+
+    def close_tab(self) -> None:
+        """Close current tab and switch to the previously tab."""
+        self.driver.close()
+        self.switch_previous_tab()
 
     def scroll_to_bottom(self) -> None:
         """Scroll to the bottom of the page."""
@@ -764,7 +778,8 @@ class Collection(Base, Sequence):
             css_probe = self.locator if self.by == By.CSS_SELECTOR else None
             digest = build_digest(
                 self.driver,
-                title="Element with given text not found",
+                title=f'Element with text not found: "{text}" '
+                f"({'contains' if partial_text else 'equals'})",
                 locator=f"{self.by}={self.locator}",
                 condition=f"text {'contains' if partial_text else 'equals'}"
                 f' "{text}"',
@@ -939,12 +954,12 @@ class ElementBase(Base):
                 digest = f"{digest}\n{msg}"
             raise ElementLocationException(digest) from None
 
-    def wait_to_disappear(self, time: int = 10, msg: str = "") -> bool | None:
+    def wait_to_disappear(self, timeout: int = 10, msg: str = "") -> bool | None:
         """
         Wait until element disappears from DOM or is hidden.
 
         Args:
-            time: Maximum wait time in seconds.
+            timeout: Maximum wait time in seconds.
             msg: Optional extra message for error context.
 
         Raises:
@@ -955,7 +970,7 @@ class ElementBase(Base):
         """
         css = self._fmt_locator()
         css_probe = self.locator if self.by == By.CSS_SELECTOR else None
-        push_step(f'wait_to_disappear("{css}", {time}s)')
+        push_step(f'wait_to_disappear("{css}", {timeout}s)')
 
         def _gone(_self: Self) -> bool:
             obj = _self._get_current_driver()
@@ -971,7 +986,7 @@ class ElementBase(Base):
                 return True
 
         try:
-            return WebDriverWait(self.driver, time).until(
+            return WebDriverWait(self.driver, timeout).until(
                 lambda driver: _gone(self)
             )
         except TimeoutException:
@@ -979,7 +994,7 @@ class ElementBase(Base):
                 self.driver,
                 title="Element(s) did not disappear",
                 locator=css,
-                wait_seconds=time,
+                wait_seconds=timeout,
                 css_for_probe=css_probe,
                 **self._digest_ctx(),
             )
@@ -1143,16 +1158,31 @@ class ElementBase(Base):
         try:
             self.scroll_into_view()
             self.lookup().click()
-        except Exception:
+        except ElementClickInterceptedException as e:
+            log.error(
+                f"[PROMIUM] Original webdriver exception: "
+                f"{type(e).__name__}: {e}"
+            )
             digest = build_digest(
                 self.driver,
-                title="Click failed",
+                title="Click failed, something is covering",
                 locator=css,
                 condition="webelement.click()",
                 css_for_probe=css_probe,
                 **self._digest_ctx(),
             )
             raise ElementLocationException(digest) from None
+        except Exception as e:
+            # print unexpect error
+            if type(e).__name__ not in {
+                PromiumException.__name__,
+                ElementLocationException.__name__,
+            }:
+                log.error(
+                    "[PROMIUM] Original webdriver exception: "
+                    f"{type(e).__name__}: {e}"
+                )
+            raise ElementLocationException(e) from None
 
     def js_click(self) -> None:
         """Perform click using JavaScript executor."""
