@@ -1,7 +1,8 @@
 import os
-import sys
-import subprocess
+import pathlib
 import shutil
+import subprocess
+
 import pytest
 
 
@@ -10,16 +11,12 @@ class EnvironmentException(Exception):
 
 
 def run_command(command):
-    print(f"Running command: {command}")
     process = subprocess.run(
         command,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
+        check=False, capture_output=True,
     )
 
     if process.returncode != 0:
-        print(process.stdout.decode())
-        print(process.stderr.decode())
         raise EnvironmentException(f"Failed to run command: {command}")
 
     return process
@@ -27,19 +24,19 @@ def run_command(command):
 
 @pytest.fixture
 def docs_fixture(scope="function"):
-    with open("version", "w") as version_file:
+    with pathlib.Path("version").open("w", encoding="utf-8") as version_file:
         version_file.write("13.666.2")
 
     yield
 
     os.chdir("/work")
-    if os.path.exists("doc/_build"):
+    if pathlib.Path("doc/_build").exists():
         shutil.rmtree("doc/_build", ignore_errors=True)
-    if os.path.exists("version"):
-        os.remove("version")
-    if os.path.exists("dist"):
+    if pathlib.Path("version").exists():
+        pathlib.Path("version").unlink()
+    if pathlib.Path("dist").exists():
         shutil.rmtree("dist", ignore_errors=True)
-    if os.path.exists("Promium.egg-info"):
+    if pathlib.Path("Promium.egg-info").exists():
         shutil.rmtree("Promium.egg-info", ignore_errors=True)
 
 
@@ -48,7 +45,7 @@ class TestEnvironment:
     @pytest.mark.skip("pass")
     def test_check_doc_command(self, docs_fixture):
         run_command(["sphinx-build", "-b", "html", "doc", "public"])
-        assert os.path.exists("public/index.html"), "Not Create doc files"
+        assert pathlib.Path("public/index.html").exists(), "Not Create doc files"
 
     @pytest.mark.env
     def test_check_publish(self, docs_fixture):
